#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#----------------------------------------------------------------------
# xoutil.fs.path
#----------------------------------------------------------------------
# Copyright (c) 2013, 2014 Merchise Autrement and Contributors
# Copyright (c) 2012 Medardo Rodríguez
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under the
# terms of the LICENCE attached (see LICENCE file) in the distribution
# package.
#
# Created on Feb 16, 2012

'''Extensions to os.path

Functions inside this module must not have side-effects on the
file-system. This module re-exports (without change) several functions from the
:mod:`os.path` standard module.

'''

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode)

import sys
from os.path import (abspath, expanduser, dirname, sep, normpath,
                     join as _orig_join)

from xoutil.functools import power as pow_
from xoutil.names import strlist as strs
__all__ = strs('abspath', 'expanduser', 'dirname', 'sep', 'normpath', 'rtrim',
               'fix_encoding', 'join', 'normalize_path',
               'shorten_module_filename', 'shorten_user')
del strs


__docstring_format__ = 'rst'
__author__ = 'manu'


# TODO: import all in "from os.path import *"

rtrim = lambda path, n: pow_(dirname, n)(normalize_path(path))
rtrim.__doc__ = """Trims the last `n` components of the pathname `path`.

This basically applies `n` times the function `os.path.dirname` to `path`.

`path` is normalized before proceeding (but not tested to exists).

Example::

    >>> rtrim('~/tmp/a/b/c/d', 3)  # doctest: +ELLIPSIS
    '.../tmp/a'

    # It does not matter if `/` is at the end
    >>> rtrim('~/tmp/a/b/c/d/', 3)  # doctest: +ELLIPSIS
    '.../tmp/a'

"""


def fix_encoding(name, encoding=None):
    '''Fix encoding of a file system resource name.

    `encoding` is ignored if `name` is already a `str`.

    '''
    if not isinstance(name, str):
        if not encoding:
            from xoutil.string import force_encoding
            encoding = force_encoding(sys.getfilesystemencoding())
        fixer = name.decode if isinstance(name, bytes) else name.encode
        return fixer(encoding)
    else:
        return name


def join(base, *extras):
    '''Join two or more pathname components, inserting '/' as needed.

    If any component is an absolute path, all previous path components
    will be discarded.

    Normalize path (after join parts), eliminating double slashes, etc.

    '''
    try:
        path = _orig_join(base, *extras)
    except:
        base = fix_encoding(base)
        extras = [fix_encoding(extra) for extra in extras]
        path = _orig_join(base, *extras)
    return normpath(path)


def normalize_path(base, *extras):
    '''Normalize path by:

    - expanding '~' and '~user' constructions.
    - eliminating double slashes
    - converting to absolute.

    '''
    # FIXME: [med] Redundant "path" in name "xoutil.fs.path.normalize_path"
    try:
        path = _orig_join(base, *extras)
    except:
        path = join(base, *extras)
    return abspath(expanduser(path))


def shorten_module_filename(filename):
    '''A filename, normally a module o package name, is shortened looking his
    head in all python path.

    '''
    path = sys.path[:]
    path.sort(lambda x, y: len(y) - len(x))
    for item in path:
        if item and filename.startswith(item):
            filename = filename[len(item):]
            if filename.startswith(sep):
                filename = filename[len(sep):]
    for item in ('__init__.py', '__init__.pyc'):
        if filename.endswith(item):
            filename = filename[:-len(item)]
            if filename.endswith(sep):
                filename = filename[:-len(sep)]
    return shorten_user(filename)


def shorten_user(filename):
    '''A filename is shortened looking for the (expantion) $HOME in his head
    and replacing it by '~'.

    '''
    home = expanduser('~')
    if filename.startswith(home):
        filename = _orig_join('~', filename[len(home):])
    return filename



# -*- coding: utf-8 -*-
#----------------------------------------------------------------------
# xoutil.data
#----------------------------------------------------------------------
# Copyright (c) 2013, 2014 Merchise Autrement and Contributors
# Copyright (c) 2009-2012 Medardo Rodríguez
# All rights reserved.
#
# Author: Medardo Rodriguez
# Contributors: see CONTRIBUTORS and HISTORY file
#
# This is free software; you can redistribute it and/or modify it under the
# terms of the LICENCE attached (see LICENCE file) in the distribution
# package.

'''Some useful Data Structures and data-related algorithms and functions.

.. deprecated:: 1.4.0 This module is completely deprecated since 1.4.0. Most of
   its contents are already deprecated.

'''


from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import as _py3_absimports)

import xoutil.collections
from xoutil.deprecation import deprecated


def adapt_exception(value, **kwargs):
    '''Like PEP-246, Object Adaptation, with ``adapt(value, Exception,
    None)``.

    If the value is not an exception is expected to be a tuple/list which
    contains an Exception type as its first item.

    '''
    isi, issc, ebc = isinstance, issubclass, Exception
    if isi(value, ebc) or isi(value, type) and issc(value, ebc):
        return value
    elif isi(value, (tuple, list)) and len(value) > 0 and issc(value[0], ebc):
        from xoutil.six import string_types as str_base
        iss = lambda s: isinstance(s, str_base)
        ecls = value[0]
        args = ((x.format(**kwargs) if iss(x) else x) for x in value[1:])
        return ecls(*args)
    else:
        return None



# -*- encoding: utf-8 -*-
#----------------------------------------------------------------------
# xoutil.textwrap
#----------------------------------------------------------------------
# Copyright (c) Merchise Autrement and Contributors
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under the
# terms of the Python Software Licence as of Python 3.3.
#
# Created on 2014-02-26

'''Text wrapping and filling.

'''

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import as _py3_abs_import)

from xoutil.modules import copy_members as _copy_python_module_members
_pm = _copy_python_module_members()
_pm_dedent = _pm.dedent
_pm_indent = getattr(_pm, 'indent', None)

del _pm, _copy_python_module_members


def dedent(text, skip_firstline=False):
    r'''Remove any common leading whitespace from every line in text.

    This can be used to make triple-quoted strings line up with the left edge
    of the display, while still presenting them in the source code in indented
    form.

    Note that tabs and spaces are both treated as whitespace, but they are not
    equal: the lines ``"    hello"`` and ``"\thello"`` are considered to have
    no common leading whitespace.

    If `skip_firstline` is True, the first line is separated from the rest of
    the body.  This helps with docstrings that follow :pep:`257`.

    .. warning:: The `skip_firstline` argument is missing in standard library.

    '''
    if skip_firstline:
        parts = text.split('\n', 1)
        if len(parts) > 1:
            subject, body = parts
        else:
            subject, body = parts[0], ''
        result = _pm_dedent(subject)
        if body:
            result += '\n' + _pm_dedent(body)
    else:
        result = _pm_dedent(text)
    return result


if not _pm_indent:
    #
    # The following is Copyright (c) of the Python Software Foundation.
    #
    def indent(text, prefix, predicate=None):
        """Adds 'prefix' to the beginning of selected lines in 'text'.

        If 'predicate' is provided, 'prefix' will only be added to the lines
        where 'predicate(line)' is True. If 'predicate' is not provided, it
        will default to adding 'prefix' to all non-empty lines that do not
        consist solely of whitespace characters.

        .. note:: Backported from Python 3.3.  In Python 3.3 this is an alias.

        """
        if predicate is None:
            def predicate(line):
                return line.strip()

        def prefixed_lines():
            for line in text.splitlines(True):
                yield (prefix + line if predicate(line) else line)
        return ''.join(prefixed_lines())



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#----------------------------------------------------------------------
# xoutil.tests.test_fs
#----------------------------------------------------------------------
# Copyright (c) 2013, 2014 Merchise Autrement and Contributors
# Copyright (c) 2011, 2012 Medardo Rodríguez
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under
# the terms of the LICENCE attached in the distribution package.
#
# Created on Feb 15, 2012

'Tests for xoutil.fs module'

import os
import sys
import unittest
import tempfile
import shutil

# Makes sure our package is always importable
# sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..')))

class TestFs(unittest.TestCase):
    def setUp(self):
        # Makes all names predictable
        pjoin = os.path.join
        self.previous_dir = os.getcwd()
        self.base = base = tempfile.mkdtemp(prefix='xoutiltests-')
        os.makedirs(pjoin(base, 'A', 'B', 'C'))
        os.makedirs(pjoin(base, 'A', 'D', 'E'))
        os.makedirs(pjoin(base, 'A', 'F'))
        self.files = files = []

        wexpected = self.walk_up_expected = pjoin(self.base, 'A')
        sentinel = tempfile.mkstemp(prefix='X', dir=wexpected)
        self.sentinel = os.path.basename(sentinel[-1])
        files.append(sentinel)   # For testing `walk_up`
        files.append(tempfile.mkstemp(prefix='M', dir=pjoin(self.base,
                                                            'A', 'B')))
        files.append(tempfile.mkstemp(prefix='P',
                                      dir=pjoin(self.base, 'A', 'B')))
        wstart = self.walk_up_start = pjoin(self.base, 'A', 'B', 'C')
        files.append(tempfile.mkstemp(prefix='z', dir=wstart))
        files.append(tempfile.mkstemp(suffix='ending', dir=pjoin(self.base,
                                                                 'A', 'D')))
        files.append(tempfile.mkstemp(prefix='Z', dir=pjoin(self.base,
                                                            'A', 'F')))

    def test_iter_files_with_regex_pattern(self):
        from xoutil.fs import iter_files
        res = list(iter_files(self.base, '(?xi)/Z'))
        self.assertEquals(2, len(res))
        self.assertIn(self.files[-3][-1], res)
        self.assertIn(self.files[-1][-1], res)

    def test_iter_files_with_maxdepth(self):
        from xoutil.fs import iter_files
        res = list(iter_files(self.base, '(?xi)/Z', maxdepth=3))
        self.assertEquals(1, len(res))
        self.assertIn(self.files[-1][-1], res)
        res = list(iter_files(self.base, '(?xi)/Z', maxdepth=2))
        self.assertEquals(0, len(res))

    def test_walk_up(self):
        from xoutil.fs import walk_up
        expected, start = self.walk_up_expected, self.walk_up_start
        sentinel = self.sentinel
        self.assertEqual(expected, walk_up(start, sentinel))

    def test_ensure_filename(self):
        from xoutil.fs import ensure_filename
        filename = os.path.join(self.base, 'en', 'sure', 'filename.txt')
        ensure_filename(filename)
        self.assert_(os.path.isfile(filename))

    def tearDown(self):
        shutil.rmtree(self.base)
        os.chdir(self.previous_dir)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#----------------------------------------------------------------------
# xoutil.tests.test_modules
#----------------------------------------------------------------------
# Copyright (c) 2013, 2014 Merchise Autrement and Contributors
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under
# the terms of the LICENCE attached in the distribution package.
#
# Created on 2013-01-28

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import as _py3_abs_imports)

import sys
import unittest

from xoutil.modules import customize, modulemethod, moduleproperty

__author__ = "Manuel Vázquez Acosta <mva.led@gmail.com>"
__date__   = "Mon Jan 28 19:32:00 2013"


class TestModulesCustomization(unittest.TestCase):
    def setUp(self):
        import testbed
        self.testbed = testbed

    def tearDown(self):
        sys.modules[self.testbed.__name__] = self.testbed

    def test_echo(self):
        import testbed
        module, created, klass = customize(testbed)
        self.assertEqual(10, module.echo(10))

    def test_module_props(self):
        @property
        def this(mod):
            return mod

        import testbed
        module, created, klass = customize(testbed, custom_attrs={'this':this})
        self.assertEqual(module, module.this)


class TestModuleDecorators(unittest.TestCase):
    def test_echo_module_level(self):
        import sys

        @modulemethod
        def echo(self, *args):
            return (self, args)

        current_module = sys.modules[__name__]
        self.assertEquals((current_module, (1, 2)), echo(1, 2))


    def test_moduleproperties(self):
        import customizetestbed as m
        self.assertIs(m, m.this)
        self.assertIs(None, m.store)
        m.store = (1, 2)
        self.assertEquals((1, 2), m.store)
        self.assertEquals((1, 2), m._store)


def test_get_module_path_by_module_object():
    import xoutil, xoutil.iterators
    from os.path import dirname, join
    from xoutil.modules import get_module_path
    top = join(dirname(dirname(__file__)), 'xoutil')
    expected = top
    assert get_module_path(xoutil) == expected

    expected = (join(top, 'iterators.py'),
                join(top, 'iterators.pyc'),
                join(top, 'iterators.pyo'))
    assert get_module_path(xoutil.iterators) in expected


def test_get_module_path_by_module_string_abs():
    from os.path import dirname, join
    from xoutil.modules import get_module_path
    top = join(dirname(dirname(__file__)), 'xoutil')
    expected = top
    assert get_module_path('xoutil') == expected

    expected = (join(top, 'iterators.py'),
                join(top, 'iterators.pyc'),
                join(top, 'iterators.pyo'))
    assert get_module_path('xoutil.iterators') in expected


def test_get_module_path_by_module_string_rel():
    import pytest
    from xoutil.modules import get_module_path
    with pytest.raises(TypeError):
        assert get_module_path('.iterators') == expected



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#----------------------------------------------------------------------
# xoutil.tests.testbed
#----------------------------------------------------------------------
# Copyright (c) 2013, 2014 Merchise Autrement and Contributors
# Copyright (c) 2012 Medardo Rodríguez
# All rights reserved.
#
# Author: Manuel Vázquez Acosta
# Contributors: see CONTRIBUTORS and HISTORY file
#
# This is free software; you can redistribute it and/or modify it under the
# terms of the LICENCE attached (see LICENCE file) in the distribution
# package.
#
# Created on Apr 29, 2012

'''
This module simply serves the purposes of the testing weaving modules
'''

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import)

__docstring_format__ = 'rst'
__author__ = 'manu'


def echo(what):
    return what


from xoutil.modules import moduleproperty, modulemethod

@moduleproperty
def this(self):
    return self


def rien():
    return 1

@modulemethod
def method(self, *args, **kwargs):
    return self, args, kwargs



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#----------------------------------------------------------------------
# xoutil.pprint
#----------------------------------------------------------------------
# Copyright (c) 2013, 2014 Merchise Autrement and Contributors
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under
# the terms of the LICENCE attached in the distribution package.
#
# Created on 2013-05-06

'''Enhanced data pretty printer.

'''

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import as _py3_abs_imports)

from xoutil.modules import copy_members as _copy_members
import pprint as _pm
_copy_members(_pm)

pprint = _pm.pprint  # Avoid IDE complaints

from xoutil.names import strlist as strs
__all__ = strs('ppformat', *getattr(_pm, '__all__', dir(_pm)))
del strs
del _pm, _copy_members


def ppformat(obj):
    '''Just like :func:`pprint` but always returns the result instead of
    writing it to a stream.

    :returns: The pretty formated text.
    :rtype: `unicode` in Python 2, `str` in Python 3.

    '''
    import io
    from xoutil.six import PY3, text_type
    if PY3:
        stream = io.StringIO()
    else:
        stream = io.BytesIO()
    pprint(obj, stream=stream)
    stream.seek(0)
    res = stream.read()
    if isinstance(res, text_type):
        return res
    else:
        from xoutil.string import safe_decode
        return safe_decode(res)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#----------------------------------------------------------------------
# tests.test_names
#----------------------------------------------------------------------
# Copyright (c) 2013, 2014 Merchise Autrement
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License (GPL) as published by the
# Free Software Foundation;  either version 2  of  the  License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#
# Created on 16 avr. 2013


from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import as _py3_abs_import)

__docstring_format__ = 'rst'
__author__ = 'med'

from xoutil.collections import OrderedSmartDict


def test_nameof():
    from xoutil.names import nameof
    from collections import OrderedDict as sorted_dict
    assert nameof(sorted_dict) == 'sorted_dict'
    assert nameof(sorted_dict, inner=True) == 'OrderedDict'
    sd = sorted_dict(x=1, y=2)
    assert nameof(sd) == 'sd'
    assert nameof(sd, typed=True) == 'sorted_dict'
    assert nameof(sd, inner=True, typed=True) == 'OrderedDict'
    s = 'foobar'
    assert nameof(s, inner=True) == 'foobar'
    # The following needs to be tested outside the assert, cause in Py3.3,
    # py.test rewrites the assert sentences and the local scope `nameof`
    # searched is not reached properly.
    passed = nameof('foobar') == 's'
    assert passed

    i = 1
    assert nameof(i) == 'i'
    assert nameof(i, inner=True) == '1'
    assert nameof(i, typed=True) == 'int'
    assert hex(id(sd)) in nameof(sd, inner=True)



def test_fullnameof():
    from xoutil.names import nameof
    from collections import OrderedDict as sorted_dict
    assert nameof(sorted_dict, full=True) == 'test_fullnameof.sorted_dict'
    assert nameof(sorted_dict, inner=True, full=True) == 'collections.OrderedDict'
    sd = sorted_dict(x=1, y=2)
    assert nameof(sd, full=True) == 'test_fullnameof.sd'
    assert nameof(sd, typed=True, full=True) == 'test_fullnameof.sorted_dict'
    assert nameof(sd, inner=True, typed=True, full=True) == 'collections.OrderedDict'


def test_fullnameof_no_rename():
    from xoutil.names import nameof
    from collections import OrderedDict
    assert nameof(OrderedDict, full=True) == 'test_fullnameof_no_rename.OrderedDict'
    assert nameof(OrderedDict, inner=True, full=True) == 'collections.OrderedDict'

def test_module_level_name():
    from xoutil.names import nameof
    assert nameof(OrderedSmartDict) == 'OrderedSmartDict'
    assert nameof(OrderedSmartDict, typed=True) == 'OrderedSmartDict'
    assert nameof(OrderedSmartDict, full=True) == 'xoutil.collections.OrderedSmartDict'



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#----------------------------------------------------------------------
# xoutil.cli
#----------------------------------------------------------------------
# Copyright (c) 2013, 2014 Merchise Autrement
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under
# the terms of the LICENCE attached in the distribution package.
#
# Created on 3 mai 2013

'''Define tools for command-line interface (CLI) applications.

CLi is a means of interaction with a computer program where the user (or
client) issues commands to the program in the form of successive lines of text
(command lines).

.. versionadded:: 1.4.1

'''

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import as _py3_abs_import)

from abc import abstractmethod, ABCMeta
from xoutil.objects import metaclass, classproperty
from .tools import command_name, program_name


__docstring_format__ = 'rst'
__author__ = 'med'



class RegistryDescriptor(object):
    '''Define a mechanism to automatically obtain all registered commands.'''

    __slots__ = [str('cache')]

    def __init__(self):
        self.cache = {}

    def __get__(self, instance, owner):
        if instance is None and owner is Command:
            if not self.cache:
                self._settle_cache(Command)
                assert self.cache.pop(command_name(Command), None) is None
            return self.cache
        else:
            if instance:
                obj = 'Instance %s of class %s' % (id(instance),
                                             type(instance).__name__)
            else:
                obj = 'Class %s' % owner.__name__
            msg = 'Only allowed in class "Command"; used invalidly from "%s"!'
            raise AttributeError(msg % obj)

    def _settle_cache(self, source, recursed=set()):
        # TODO: Convert check based in argument "recursed" in a decorator
        name = source.__name__
        if name not in recursed:
            recursed.add(name)
            sub_commands = source.__subclasses__()
            sub_commands.extend(getattr(source, '_abc_registry', ()))
            cmds = getattr(source, '__commands__', None)
            if cmds:
                sub_commands.extend(cmds())
            if sub_commands:
                for cmd in sub_commands:
                    self._settle_cache(cmd, recursed=recursed)
            else:   # Only branch commands are OK to execute
                self.cache[command_name(source)] = source
        else:
            raise ValueError('Reused class name "%s"!' % name)


class Command(metaclass(ABCMeta)):
    '''A command base registry.

    There are several methods to register new commands:

      * Inheriting from this class
      * Using the ABC mechanism of `register` virtual subclasses.
      * Registering a class with the method "__commands__" defined.

    If the method "__commands__" is used, it must be a class or static method.

    Command names are calculated as class names in lower case inserting a
    hyphen before each new capital letter. For example "MyCommand" will be
    used as "my-command".

    Each command could include its own argument parser, but it isn't used
    automatically, all arguments will be passed as a single parameter to
    :meth:`run` removing the command when obtained from "sys.argv".

    '''
    __default_command__ = None

    def __str__(self):
        return command_name(self)

    def __repr__(self):
        return '<command: %s>' % command_name(self)

    @classproperty
    def registry(cls):
        '''Obtain all registered commands.'''
        if cls is Command:
            name = '__registry__'
            res = getattr(cls, name, {})
            if not res:
                cls._settle_cache(res, Command)
                assert res.pop(command_name(Command), None) is None
                cls._check_help(res)
                setattr(cls, name, res)
            return res
        else:
            msg = ('Invalid class "%s" for use this property, only allowed in '
                   '"Command"!')
            raise TypeError(msg % cls.__name__)
        # = RegistryDescriptor()

    @abstractmethod
    def run(self, args=[]):
        '''Must return a valid value for "sys.exit"'''
        raise NotImplemented

    @classmethod
    def set_default_command(cls, cmd=None):
        '''A default command can be defined for call it when no one is
        specified.

        A command is detected when its name appears as the first command-line
        argument.

        To specify a default command, use this method with the command as a
        string (the command name) or the command class.

        If the command is specified, then the calling class is the selected
        one.

        For example::

            >>> Command.set_default_command('server')  # doctest: +SKIP
            >>> Server.set_default_command()           # doctest: +SKIP
            >>> Command.set_default_command(Server)    # doctest: +SKIP

        '''
        if cls is Command:
            if cmd is not None:
                from xoutil.six import string_types as text
                name = cmd if isinstance(cmd, text) else command_name(cmd)
            else:
                raise ValueError('missing command specification!')
        else:
            if cmd is None:
                name = command_name(cls)
            else:
                msg = 'redundant command specification: "%s" and "%s"!'
                raise ValueError(msg % (cls, cmd))
        Command.__default_command__ = name

    @staticmethod
    def _settle_cache(target, source, recursed=set()):
        '''`target` is a mapping to store result commands'''
        # TODO: Convert check based in argument "recursed" in a decorator
        from xoutil.names import nameof
        name = nameof(source, inner=True, full=True)
        if name not in recursed:
            recursed.add(name)
            sub_commands = type.__subclasses__(source)
            sub_commands.extend(getattr(source, '_abc_registry', ()))
            cmds = getattr(source, '__commands__', None)
            if cmds:
                from collections import Iterable
                if not isinstance(cmds, Iterable):
                    cmds = cmds()
                sub_commands.extend(cmds)
            if sub_commands:
                for cmd in sub_commands:
                    Command._settle_cache(target, cmd, recursed=recursed)
            else:   # Only branch commands are OK to execute
                from types import MethodType
                assert isinstance(source.run, MethodType)
                target[command_name(source)] = source
        else:
            raise ValueError('Reused class "%s"!' % name)

    @staticmethod
    def _check_help(target):
        '''Check that correct help command is present.'''
        name = HELP_NAME
        hlp = target[name]
        if hlp is not Help and not getattr(hlp, '__overwrite__', False):
            target[name] = Help


class Help(Command):
    '''Show all commands

    Define the class attribute `__order__` to sort commands in special command
    "help".

    Commands could define its help in the first line of a sequence of
    documentations until found:

      - command class,
      - "run" method,
      - definition module.

    This command could not be overwritten unless using the class attribute:

       __override__ = True

    '''

    __order__ = -9999

    @classmethod
    def get_arg_parser(cls):
        '''This is an example on how to build local argument parser.

        Use class method "get

        '''
        res = getattr(cls, '_arg_parser')
        if not res:
            from argparse import ArgumentParser
            res = ArgumentParser()
            cls._arg_parser = res
        return res

    def run(self, args=[]):
        print('The most commonly used "%s" commands are:' % program_name())
        cmds = Command.registry
        ordered = [(getattr(cmds[cmd], '__order__', 0), cmd) for cmd in cmds]
        ordered.sort()
        max_len = len(max(ordered, key=lambda x: len(x[1]))[1])
        for _, cmd in ordered:
            cmd_class = cmds[cmd]
            doc = self._strip_doc(cmd_class.__doc__)
            if not doc:
                doc = self._strip_doc(cmd_class.run.__doc__)
            if not doc:
                import sys
                mod_name = cmd_class.__module__
                module = sys.modules.get(mod_name, None)
                if module:
                    doc = self._strip_doc(module.__doc__)
                    doc = '"%s"' % (doc if doc else mod_name)
                else:
                    doc = '"%s"' % mod_name
            head = ' '*3 + cmd + ' '*(2 + max_len - len(cmd))
            print(head, doc)

    @staticmethod
    def _strip_doc(doc):
        if doc:
            doc = str('%s' % doc).strip()
            return str(doc.strip().split('\n')[0].strip('''.'" \t\n\r'''))
        else:
            return ''


HELP_NAME = command_name(Help)

# TODO: Create "xoutil.config" here

del RegistryDescriptor
del abstractmethod, ABCMeta
del metaclass, classproperty



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#----------------------------------------------------------------------
# xoutil._meta3
#----------------------------------------------------------------------
# Copyright (c) 2013, 2014 Merchise Autrement and Contributors
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under
# the terms of the LICENCE attached in the distribution package.
#
# Created on 2013-04-29

'''Implements the meta() function using the Py3k syntax.

'''

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import as _py3_abs_imports)

from xoutil.six import PY3
assert PY3, 'This module should be loaded in Py3k only'


def metaclass(meta):
    class base:
        pass

    class inner_meta(meta.__base__):
        def __new__(cls, name, bases, attrs):
            if name != '__inner__':
                bases = tuple(b for b in bases if not issubclass(b, base))
                if not bases:
                    bases = (object,)
                return meta(name, bases, attrs)
            else:
                return type.__new__(cls, name, bases, attrs)

        def __init__(self, name, bases, attrs):
            pass

    # Hide this stuff from Python 2.7 compiler so several static analysis tools
    # (like pylint and other) don't complain about a SyntaxError.
    eval(compile('class __inner__(base, metaclass=inner_meta):pass',
                 __file__, 'exec'))

    return locals()['__inner__']



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#----------------------------------------------------------------------
# xoutil.cli.tools
#----------------------------------------------------------------------
# Copyright (c) 2013, 2014 Merchise Autrement and Contributors
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under
# the terms of the LICENCE attached in the distribution package.
#
# Created on 5 mai 2013

'''Utilities for command-line interface (CLI) applications.

'''

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import as _py3_abs_import)

__docstring_format__ = 'rst'
__author__ = 'med'


def program_name():
    '''Calculate the program name from "sys.argv[0]".'''
    import sys
    from os.path import basename
    return basename(sys.argv[0])


def command_name(cls):
    '''Command names are calculated as class names in lower case inserting a
    hyphen before each new capital letter. For example "MyCommand" will be
    used as "my-command".

    It's defined as an external function because a class method don't apply to
    minimal commands (those with only the "run" method).

    Example::

        >>> class SomeCommand(object):
        ...     pass

        >>> command_name(SomeCommand) == 'some-command'
        True

    If the command class has an attribute `command_cli_name`, this will be
    used instead::

        >>> class SomeCommand(object):
        ...    command_cli_name = 'adduser'

        >>> command_name(SomeCommand) == 'adduser'
        True

    It's an error to have a non-string `command_cli_name` attribute::

        >>> class SomeCommand(object):
        ...    command_cli_name = None

        >>> command_name(SomeCommand)  # doctest: +ELLIPSIS
        Traceback (most recent call last):
           ...
        TypeError: Attribute 'command_cli_name' must be a string.

    '''
    Unset = object()
    res = getattr(cls, 'command_cli_name', Unset)
    if res is not Unset:
        from xoutil.six import text_type
        if not isinstance(res, text_type):
            raise TypeError("Attribute 'command_cli_name' must be a string.")
    else:
        from io import StringIO
        from xoutil.string import safe_decode
        buf = StringIO()
        start = True
        for letter in cls.__name__:
            if letter.isupper():
                if not start:
                    buf.write(safe_decode('-'))
                letter = letter.lower()
            buf.write(safe_decode(letter))
            start = False
        buf.flush()
        res = buf.getvalue()
        buf.close()
    return res



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#----------------------------------------------------------------------
# xoutil.tests.test_metaclass
#----------------------------------------------------------------------
# Copyright (c) 2013, 2014 Merchise Autrement and Contributors
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under
# the terms of the LICENCE attached in the distribution package.
#
# Created on 2013-05-06

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import as _py3_abs_imports)

__author__ = "Manuel Vázquez Acosta <mva.led@gmail.com>"
__date__ = "Mon May  6 15:55:00 2013"


def test_basic_inline_metaclass():
    from xoutil.objects import metaclass

    class Meta(type):
        pass

    class Base(metaclass(Meta)):
        pass

    class Entity(Base):
        pass

    assert type(Base) is Meta
    assert type(Entity) is Meta
    assert Entity.__base__ is Base
    assert Base.__base__ is object


def test_no_double_registration_with_inlinemetaclass():
    import sys
    from xoutil.objects import metaclass
    py32 = sys.version_info >= (3, 2)

    class RegisteringType(type):
        classes = []
        def __new__(cls, name, bases, attrs):
            res = super(RegisteringType, cls).__new__(cls, name, bases, attrs)
            cls.classes.append(res)
            return res

    class Base(metaclass(RegisteringType)):
        pass

    class SubType(RegisteringType):
        pass

    class Egg(metaclass(SubType), Base):
        pass

    assert len(RegisteringType.classes) == 2

    class Spam(Base, metaclass(SubType)):
        'Like "Egg" but registered twice in Python 2.x.'

    assert len(RegisteringType.classes) == (3 if py32 else 4)


def test_inlinemetaclass_decorator_with_slots():
    from xoutil.objects import metaclass
    from xoutil.types import MemberDescriptorType

    class Meta(type):
        pass

    class Base(metaclass(Meta)):
        __slots__ = 'attr'

    class Ok(metaclass(Meta)):
        def __init__(self, **kwargs):
            self.__dict__ = kwargs

        @classmethod
        def clmethod(cls):
            return cls

        @staticmethod
        def stmethod(echo):
            return echo

        def echo(self, echo):
            return self, echo

    assert isinstance(Base.attr, MemberDescriptorType)
    assert isinstance(Base, Meta)
    assert isinstance(Ok, Meta)

    b = Base()
    b.attr = 1
    try:
        b.another = 2
        assert False, 'Should have raised AttributeError'
    except AttributeError:
        pass
    except:
        assert False, 'Should have raised AttributeError'

    ok = Ok(name='ok')
    assert ok.stmethod(ok) == ok
    assert ok.clmethod() == Ok
    assert ok.echo(1) == (ok, 1)
    assert ok.name == 'ok'
